<?php


namespace Addons\TemplateMsg\Services;


use HttpCurl;
use Services\Service;
use Session;

class WechatApi extends Service
{
    /**
     * 获取模版库里面的模版
     * @param $tpl_key
     * @return array
     */
    public function get_tpl_info($tpl_key,$wid)
    {
        $access_token = WX_getAccessToken($wid);
        // var_dump($access_token);die;
        if (!$access_token) {
            return error('access_token不存在');
        }
        $params=http_build_query([
            "access_token"=>$access_token,
            'tid'=>$tpl_key
        ]);
        $return=HttpCurl::get('https://api.weixin.qq.com/wxaapi/newtmpl/getpubtemplatekeywords?'.$params,[]);
        $resp_arr=json_decode($return,true);
        if(0===$resp_arr['errcode']){
            return success($resp_arr['errmsg'],$resp_arr);
        }else{
            return error($resp_arr['errmsg'],$resp_arr);
        }

    }

    /**
     * 组合模版关键字添加到模版中
     * @param $tpl_key
     * @param array $keyword_ids
     * @return array
     */
    public function add_template($wid,$tpl_key,array $keyword_ids,$desc){
        $access_token = WX_getAccessToken($wid);
        // var_dump($access_token);die;
        if (!$access_token) {
            return error('access_token不存在');
        }
        $params=http_build_query([
            "access_token"=>$access_token,
            'tid'=>$tpl_key,
            'kidList'=>$keyword_ids,
            "sceneDesc"=>$desc
        ]);
        $return=HttpCurl::post('https://api.weixin.qq.com/wxaapi/newtmpl/addtemplate?access_token='.$access_token,$params);
        $resp_arr=json_decode($return,true);
        if(0===$resp_arr['errcode']){
            return success($resp_arr['errmsg'],$resp_arr);
        }else{
            return error($resp_arr['errmsg'],$resp_arr);
        }
    }

    /**
     * 获取小程序账号的类目
     * @param $wid
     * @return array
     */
    public function getCategory($wid){
        $access_token = WX_getAccessToken($wid);
        // var_dump($access_token);die;
        if (!$access_token) {
            return error('access_token不存在');
        }

        $return=HttpCurl::get('https://api.weixin.qq.com/wxaapi/newtmpl/getcategory?access_token='.$access_token,array());
        $resp_arr=json_decode($return,true);
        if(0===$resp_arr['errcode']){
            return success($resp_arr['errmsg'],$resp_arr);
        }else{
            return error($resp_arr['errmsg'],$resp_arr);
        }
    }

    /**
     * 获取当前类目下模板列表
     * @param $wid
     * @param $ids
     * @param $start
     * @param $limit
     * @return array
     */
    public function getPubTplList($wid,$ids,$start,$limit){
        $access_token = WX_getAccessToken($wid);
        // var_dump($access_token);die;
        if (!$access_token) {
            return error('access_token不存在');
        }
        $params=http_build_query([
            'ids'=>is_string($ids) ? $ids : implode(",",$ids) ,
            "start"=>$start,
            "limit"=>$limit,
            "access_token"=>$access_token
        ]);
        $return=HttpCurl::get('https://api.weixin.qq.com/wxaapi/newtmpl/getpubtemplatetitles?'.$params,[]);
        $resp_arr=json_decode($return,true);
        if(0===$resp_arr['errcode']){
            return success($resp_arr['errmsg'],$resp_arr);
        }else{
            return error($resp_arr['errmsg'],$resp_arr);
        }
    }

    /**
     * 在用户授权之后的回调Url中，根据返回的临时auth_code，获取authorizer_access_token和authorizer_refresh_token并妥善保存
     * 或者直接获取最新的authorizer_access_token
     * authorizer_access_token过期的时候自动刷新一下
     * @param null $auth_code
     * @return bool
     */
    function get_authorizer_access_token($auth_code = null) {
        //先查找当前登录的用户
        $wid = Session::get('wid');//当前登录的用户id
        $pub = (new \Model('pubs'))->find(['id' => $wid]);
        //参数是否存在
        if (!$auth_code) {
            //直接获取存储的authorizer_access_token
            if (intval(strtotime($pub->token_create_time)) + 7000 < time()) {
                //当前存储的令牌已经失效
                //发起令牌刷新请求
                //获取cat令牌
                $component_access_token = $this->get_component_access_token();
                if (!$component_access_token) {
                    return false;
                }
                $admin = get_user_agency();
                //发起刷新请求
                diy_log([
                    "component_appid"          => $admin->app_id,
                    "authorizer_appid"         => $pub->authorizer_appid,
                    "authorizer_refresh_token" => $pub->authorizer_refresh_token,
                ]);
                $body = HttpCurl::post(
                    "https://api.weixin.qq.com/cgi-bin/component/api_authorizer_token?component_access_token=" . $component_access_token,
                    json_encode([
                        "component_appid"          => $admin->app_id,
                        "authorizer_appid"         => $pub->authorizer_appid,
                        "authorizer_refresh_token" => $pub->authorizer_refresh_token,
                    ])
                );
                $resp_arr = json_decode($body, 1);
                if (array_key_exists('authorizer_access_token', $resp_arr)) {
                    //成功获取
                    diy_log($resp_arr);
                    $pub->authorizer_access_token = $resp_arr['authorizer_access_token'];
                    $pub->authorizer_refresh_token = $resp_arr['authorizer_refresh_token'];
                    $pub->token_create_time = date('Y-m-d H:i:s', time());
                    $pub->save();
                    return $pub->authorizer_access_token;
                } else {
                    diy_log($resp_arr);
                   // rebind();
                    //获取失败
                    return false;
                }
            }
            //未失效
            return $pub->authorizer_access_token;
        }
        //第一次授权，回调url中使用
        $component_access_token = $this->get_component_access_token();
        if (!$component_access_token) {
            return false;
        }
        $admin = get_user_agency();
        $body = HttpCurl::post(
            "https://api.weixin.qq.com/cgi-bin/component/api_query_auth?component_access_token=" . $component_access_token,
            json_encode([
                "component_appid"    => $admin->app_id,
                "authorization_code" => $auth_code,
            ])
        );
        $resp_arr = json_decode($body, 1);
        if (array_key_exists('authorization_info', $resp_arr)) {
            //成功获取
            diy_log($resp_arr);
            $pub->authorizer_access_token = $resp_arr['authorization_info']['authorizer_access_token'];//权限令牌
            $pub->authorizer_refresh_token = $resp_arr['authorization_info']['authorizer_refresh_token'];//刷新令牌
            $pub->authorizer_appid = $resp_arr['authorization_info']['authorizer_appid'];//授权方appid
            $pub->token_create_time = date('Y-m-d H:i:s', time());//令牌创建时间
            $pub->save();
            $wxpay = (new \Model('vshop_weixin_pay'))->find(['wid' => $wid]);
            if (!$wxpay->id) {
                $wxpay = (new \Model('vshop_weixin_pay'));
                $wxpay->wid = $wid;
            }
            $wxpay->AppId = $resp_arr['authorization_info']['authorizer_appid'];//授权方appid
            $wxpay->save();
            return $pub->authorizer_access_token;
        } else {
            diy_log($resp_arr);
            //获取失败
            return false;
        }

    }
    /**
     * 获取开放平台的api凭证
     * Created by PhpStorm.
     * User: Jeffrey
     * Date: 2017/12/11
     * Time: 20:34
     */
    function get_component_access_token() {
        $admin = get_user_agency();
        $cat = (new \Model('component_access_token'))->find(['app_id' => $admin->app_id]);
        if ($cat->id) {
            if (intval(strtotime($cat->create_time)) + 7000 > time()) {
                //如果有未过期的ComponentAccessToken
                return $cat->component_access_token;
            }
            //存在却已过期就往下走
        } else {
            //不存在就先实例化一个
            $cat = new \Model('component_access_token');
            $cat->app_id = $admin->app_id;
        }
        // 重新请求
        $ticket = (new \Model('component_verify_ticket'))->find(['app_id' => $admin->app_id]);
        $body = HttpCurl::post(
            "https://api.weixin.qq.com/cgi-bin/component/api_component_token",
            json_encode([
                "component_appid"         => $admin->app_id,
                "component_appsecret"     => $admin->app_secret,
                "component_verify_ticket" => $ticket->component_verify_ticket,
            ])
        );
        $resp_arr = json_decode($body, 1);
        if (array_key_exists('component_access_token', $resp_arr)) {
            //请求component_access_token成功
            $cat->component_access_token = $resp_arr['component_access_token'];
            $cat->create_time = date('Y-m-d H:i:s', time());
            $cat->save();
            return $cat->component_access_token;
        } else {
            diy_log($resp_arr);
            return false;
            // return 'xxx';
        }
    }

}
